import openai
import json
from termcolor import colored
import tenacity
import wikipediaapi

model = "gpt-4-0613"

#exponantial backoff
@tenacity.retry(    wait=tenacity.wait_exponential(multiplier=1, min=4, max=10),    stop=tenacity.stop_after_attempt(5),    reraise=True, )
def return_code(model, user_input):
    
    
    response = openai.ChatCompletion.create(
        model=model,
        messages=[{"role": "system", "content": "You return wikipedia article titles based on user input. clarify and simplify the user input to match the wikipedia article title"},
            {"role": "user", "content": f"{user_input}"}],
        # stream=True,
        functions=[
            {
                "name": "get_wikipedia_article",
                "description": "This function takes in 'wikipedia article' argument and retrives the article from wikipedia",
                "parameters": {
                    "type": "object",
                    "properties": {
                        "wikipedia article" : {
                            "type": "string",
                            "description": "name of the wikipedia article based on user input",
                        },
                    },
                    "required": ["wikipedia article"],
                    }
            }

        ],
        function_call={"name": "get_wikipedia_article"}
    )

    message = response.choices[0].message
    

    if message.get("function_call"):
        function_args = json.loads(message["function_call"]["arguments"]) 
        return function_args
    
    if message["content"]:
        print(colored(message["content"], "green"))

while True:

    user_input = input("Enter user input: ")
    function_args = return_code(model, user_input)

# make an api call to wikipedia api to get the article
    wiki_wiki = wikipediaapi.Wikipedia('en')
    page_py = wiki_wiki.page(function_args["wikipedia article"])
    print(page_py.summary)
